@@ -36,8 +36,6 @@
 #include "chipdrivers.h"
 #include "erasure_layout.h"
 
-static bool use_legacy_erase_path = false;
-
 const char flashrom_version[] = FLASHROM_VERSION;
 
 static const struct programmer_entry *programmer = NULL;
@@ -486,14 +484,15 @@ int check_block_eraser(const struct flashctx *flash, int k, int log)
 
 	if (flash->mst->buses_supported & BUS_SPI) {
 		const uint8_t *opcode = spi_get_opcode_from_erasefn(eraser.block_erase);
-		for (int i = 0; opcode[i]; i++) {
-			if (!spi_probe_opcode(flash, opcode[i])) {
-				if (log)
-					msg_cdbg("block erase function and layout found "
-						 "but SPI master doesn't support the function. ");
-				return 1;
+		if (opcode)
+			for (int i = 0; opcode[i]; i++) {
+				if (!spi_probe_opcode(flash, opcode[i])) {
+					if (log)
+						msg_cdbg("block erase function and layout found "
+							 "but SPI master doesn't support the function. ");
+					return 1;
+				}
 			}
-		}
 	}
 	// TODO: Once erase functions are annotated with allowed buses, check that as well.
 	return 0;
@@ -1160,25 +1159,18 @@ int probe_flash(struct registered_master *mst, int startchip, struct flashctx *f
 		 * been found on this interface.
 		 */
 		if (startchip == 0 && flash->chip->model_id == SFDP_DEVICE_ID) {
-			msg_cinfo("===\n"
-				  "SFDP has autodetected a flash chip which is "
-				  "not natively supported by flashrom yet.\n");
+			msg_cinfo("===\nSFDP has autodetected a flash chip.\n");
 			if (count_usable_erasers(flash) == 0)
 				msg_cinfo("The standard operations read and "
-					  "verify should work, but to support "
-					  "erase, write and all other "
-					  "possible features");
+					  "verify should work, but support for "
+					  "erase and write needs to be added manually.\n");
 			else
 				msg_cinfo("All standard operations (read, "
-					  "verify, erase and write) should "
-					  "work, but to support all possible "
-					  "features");
+					  "verify, erase and write) should work.\n");
 
-			msg_cinfo(" we need to add them manually.\n"
-				  "You can help us by mailing us the output of the following command to "
-				  "flashrom@flashrom.org:\n"
-				  "'flashrom -VV [plus the -p/--programmer parameter]'\n"
-				  "Thanks for your help!\n"
+			msg_cinfo("Additionally, flashrom supports RPMC commands via SFDP autodetection.\n"
+				  "We may add support for more features via SFDP in future.\n"
+				  "If you are interested, join us on the mailing list https://flashrom.org/contact.html#mailing-list-1\n"
 				  "===\n");
 		}
 
@@ -1226,6 +1218,74 @@ notfound:
 	return chip - flashchips;
 }
 
+static void setup_progress_from_layout(struct flashctx *flashctx,
+				       enum flashrom_progress_stage stage)
+{
+	if (!flashctx->progress_callback)
+		return;
+
+	const struct flashrom_layout *const flash_layout = get_layout(flashctx);
+
+	size_t total = 0;
+	const struct romentry *entry = NULL;
+	while ((entry = layout_next_included(flash_layout, entry))) {
+		const struct flash_region *region = &entry->region;
+		total += region->end - region->start + 1;
+	}
+
+	init_progress(flashctx, stage, total);
+}
+
+
+static void setup_progress_from_layout_and_diff(struct flashctx *flashctx,
+						const void *have,
+						const void *want,
+						enum flashrom_progress_stage stage)
+{
+	if (!flashctx->progress_callback)
+		return;
+
+	const struct flashrom_layout *flash_layout = get_layout(flashctx);
+	const size_t page_size = flashctx->chip->page_size;
+
+	size_t total = 0;
+
+	const struct romentry *entry = NULL;
+	while ((entry = layout_next_included(flash_layout, entry))) {
+		const struct flash_region *region = &entry->region;
+
+		if (stage == FLASHROM_PROGRESS_ERASE) {
+			size_t offset;
+			for (offset = region->start; offset <= region->end; offset += page_size) {
+				const size_t len = min(page_size, region->end + 1 - offset);
+
+				if (need_erase(have, want, len, flashctx->chip->gran, ERASED_VALUE(flashctx)))
+					total += len;
+			}
+		}
+
+		if (stage == FLASHROM_PROGRESS_WRITE) {
+			unsigned int start = region->start;
+			unsigned int len;
+			while ((len = get_next_write(have + start, want + start,
+						     region->end + 1 - start, &start, flashctx->chip->gran))) {
+				start += len;
+				total += len;
+			}
+
+			if (flashctx->chip->feature_bits & FEATURE_NO_ERASE)
+				/* For chips with FEATURE_NO_ERASE erase op is running as write under the hood.
+				 * So typical write, which usually consists of erasing and then writing,
+				 * would be writing and then writing again. The planned total length for the
+				 * progress indicator for write is double. */
+				total *= 2;
+		}
+	}
+
+	init_progress(flashctx, stage, total);
+}
+
+
 /**
  * @brief Reads the included layout regions into a buffer.
  *
@@ -1242,6 +1302,8 @@ static int read_by_layout(struct flashctx *const flashctx, uint8_t *const buffer
 	const struct flashrom_layout *const layout = get_layout(flashctx);
 	const struct romentry *entry = NULL;
 
+	setup_progress_from_layout(flashctx, FLASHROM_PROGRESS_READ);
+
 	while ((entry = layout_next_included(layout, entry))) {
 		const struct flash_region *region = &entry->region;
 		const chipoff_t region_start	= region->start;
@@ -1329,251 +1391,7 @@ static int selfcheck_eraseblocks(const struct flashchip *chip)
 	return ret;
 }
 
-typedef int (*erasefn_t)(struct flashctx *, unsigned int addr, unsigned int len);
-/**
- * @private
- *
- * For read-erase-write, `curcontents` and `newcontents` shall point
- * to buffers of the chip's size. Both are supposed to be prefilled
- * with at least the included layout regions of the current flash
- * contents (`curcontents`) and the data to be written to the flash
- * (`newcontents`).
- *
- * For erase, `curcontents` and `newcontents` shall be NULL-pointers.
- *
- * The `chipoff_t` values are used internally by `walk_by_layout()`.
- */
-struct walk_info {
-	uint8_t *curcontents;
-	const uint8_t *newcontents;
-	chipoff_t region_start;
-	chipoff_t region_end;
-	chipoff_t erase_start;
-	chipoff_t erase_end;
-};
-/* returns 0 on success, 1 to retry with another erase function, 2 for immediate abort */
-typedef int (*per_blockfn_t)(struct flashctx *, const struct walk_info *, erasefn_t, bool *);
-
-static int walk_eraseblocks(struct flashctx *const flashctx,
-			    struct walk_info *const info,
-			    const size_t erasefunction, const per_blockfn_t per_blockfn,
-			    bool *all_skipped)
-{
-	int ret;
-	size_t i, j;
-	bool first = true;
-	struct block_eraser *const eraser = &flashctx->chip->block_erasers[erasefunction];
-
-	info->erase_start = 0;
-	for (i = 0; i < NUM_ERASEREGIONS; ++i) {
-		/* count==0 for all automatically initialized array
-		   members so the loop below won't be executed for them. */
-		for (j = 0; j < eraser->eraseblocks[i].count; ++j, info->erase_start = info->erase_end + 1) {
-			info->erase_end = info->erase_start + eraser->eraseblocks[i].size - 1;
-
-			/* Skip any eraseblock that is completely outside the current region. */
-			if (info->erase_end < info->region_start)
-				continue;
-			if (info->region_end < info->erase_start)
-				break;
-
-			/* Print this for every block except the first one. */
-			if (first)
-				first = false;
-			else
-				msg_cdbg(", ");
-			msg_cdbg("0x%06"PRIx32"-0x%06"PRIx32":", info->erase_start, info->erase_end);
-
-			erasefunc_t *erase_func = lookup_erase_func_ptr(eraser);
-			ret = per_blockfn(flashctx, info, erase_func, all_skipped);
-			if (ret)
-				return ret;
-		}
-		if (info->region_end < info->erase_start)
-			break;
-	}
-	msg_cdbg("\n");
-	return 0;
-}
-
-static int walk_by_layout(struct flashctx *const flashctx, struct walk_info *const info,
-			  const per_blockfn_t per_blockfn, bool *all_skipped)
-{
-	const struct flashrom_layout *const layout = get_layout(flashctx);
-	const struct romentry *entry = NULL;
-
-	*all_skipped = true;
-	msg_cinfo("Erasing and writing flash chip... ");
-
-	while ((entry = layout_next_included(layout, entry))) {
-		const struct flash_region *region = &entry->region;
-		info->region_start = region->start;
-		info->region_end   = region->end;
-
-		size_t j;
-		int error = 1; /* retry as long as it's 1 */
-		for (j = 0; j < NUM_ERASEFUNCTIONS; ++j) {
-			if (j != 0)
-				msg_cinfo("Looking for another erase function.\n");
-			msg_cdbg("Trying erase function %zi... ", j);
-			if (check_block_eraser(flashctx, j, 1))
-				continue;
-
-			error = walk_eraseblocks(flashctx, info, j, per_blockfn, all_skipped);
-			if (error != 1)
-				break;
-
-			if (info->curcontents) {
-				msg_cinfo("Reading current flash chip contents... ");
-				if (read_by_layout(flashctx, info->curcontents)) {
-					/* Now we are truly screwed. Read failed as well. */
-					msg_cerr("Can't read anymore! Aborting.\n");
-					/* We have no idea about the flash chip contents, so
-					   retrying with another erase function is pointless. */
-					error = 2;
-					break;
-				}
-				msg_cinfo("done. ");
-			}
-		}
-		if (error == 1)
-			msg_cinfo("No usable erase functions left.\n");
-		if (error) {
-			msg_cerr("FAILED!\n");
-			return 1;
-		}
-	}
-	if (*all_skipped)
-		msg_cinfo("\nWarning: Chip content is identical to the requested image.\n");
-	msg_cinfo("Erase/write done.\n");
-	return 0;
-}
-
-static int erase_block(struct flashctx *const flashctx,
-		       const struct walk_info *const info, const erasefn_t erasefn,
-		       bool *all_skipped)
-{
-	const unsigned int erase_len = info->erase_end + 1 - info->erase_start;
-	const bool region_unaligned = info->region_start > info->erase_start ||
-				      info->erase_end > info->region_end;
-	uint8_t *backup_contents = NULL, *erased_contents = NULL;
-	int ret = 2;
-
-	/*
-	 * If the region is not erase-block aligned, merge current flash con-
-	 * tents into a new buffer `backup_contents`.
-	 */
-	if (region_unaligned) {
-		backup_contents = malloc(erase_len);
-		erased_contents = malloc(erase_len);
-		if (!backup_contents || !erased_contents) {
-			msg_cerr("Out of memory!\n");
-			ret = 1;
-			goto _free_ret;
-		}
-		memset(backup_contents, ERASED_VALUE(flashctx), erase_len);
-		memset(erased_contents, ERASED_VALUE(flashctx), erase_len);
-
-		msg_cdbg("R");
-		/* Merge data preceding the current region. */
-		if (info->region_start > info->erase_start) {
-			const chipoff_t start	= info->erase_start;
-			const chipsize_t len	= info->region_start - info->erase_start;
-			if (read_flash(flashctx, backup_contents, start, len)) {
-				msg_cerr("Can't read! Aborting.\n");
-				goto _free_ret;
-			}
-		}
-		/* Merge data following the current region. */
-		if (info->erase_end > info->region_end) {
-			const chipoff_t start     = info->region_end + 1;
-			const chipoff_t rel_start = start - info->erase_start; /* within this erase block */
-			const chipsize_t len      = info->erase_end - info->region_end;
-			if (read_flash(flashctx, backup_contents + rel_start, start, len)) {
-				msg_cerr("Can't read! Aborting.\n");
-				goto _free_ret;
-			}
-		}
-	}
-
-	ret = 1;
-	*all_skipped = false;
-
-	msg_cdbg("E");
-
-	if (!flashctx->flags.skip_unwritable_regions) {
-		if (check_for_unwritable_regions(flashctx, info->erase_start, erase_len))
-			goto _free_ret;
-	}
-
-	unsigned int len;
-	for (unsigned int addr = info->erase_start; addr < info->erase_start + erase_len; addr += len) {
-		struct flash_region region;
-		get_flash_region(flashctx, addr, &region);
-
-		len = min(info->erase_start + erase_len, region.end + 1) - addr;
-
-		if (region.write_prot) {
-			msg_gdbg("%s: cannot erase inside %s region (%#08"PRIx32"..%#08"PRIx32"), skipping range (%#08x..%#08x).\n",
-				 __func__, region.name, region.start, region.end, addr, addr + len - 1);
-			free(region.name);
-			continue;
-		}
-
-		msg_gdbg("%s: %s region (%#08"PRIx32"..%#08"PRIx32") is writable, erasing range (%#08x..%#08x).\n",
-			 __func__, region.name, region.start, region.end, addr, addr + len - 1);
-		free(region.name);
-
-		if (erasefn(flashctx, addr, len))
-			goto _free_ret;
-		if (check_erased_range(flashctx, addr, len)) {
-			msg_cerr("ERASE FAILED!\n");
-			goto _free_ret;
-		}
-	}
-
-
-	if (region_unaligned) {
-		unsigned int starthere = 0, lenhere = 0, writecount = 0;
-		/* get_next_write() sets starthere to a new value after the call. */
-		while ((lenhere = get_next_write(erased_contents + starthere, backup_contents + starthere,
-						 erase_len - starthere, &starthere, flashctx->chip->gran))) {
-			if (!writecount++)
-				msg_cdbg("W");
-			/* Needs the partial write function signature. */
-			if (write_flash(flashctx, backup_contents + starthere,
-						  info->erase_start + starthere, lenhere))
-				goto _free_ret;
-			starthere += lenhere;
-		}
-	}
-
-	ret = 0;
-
-_free_ret:
-	free(erased_contents);
-	free(backup_contents);
-	return ret;
-}
-
-/**
- * @brief Erases the included layout regions.
- *
- * If there is no layout set in the given flash context, the whole chip will
- * be erased.
- *
- * @param flashctx Flash context to be used.
- * @return 0 on success,
- *	   1 if all available erase functions failed.
- */
-static int erase_by_layout_legacy(struct flashctx *const flashctx)
-{
-	struct walk_info info = { 0 };
-	bool all_skipped = true;
-	return walk_by_layout(flashctx, &info, &erase_block, &all_skipped);
-}
-
-static int erase_by_layout_new(struct flashctx *const flashctx)
+static int erase_by_layout(struct flashctx *const flashctx)
 {
 	bool all_skipped = true;
 	const uint32_t flash_size = flashctx->chip->total_size * 1024;
@@ -1598,6 +1416,9 @@ static int erase_by_layout_new(struct flashctx *const flashctx)
 	memset(curcontents, ~ERASED_VALUE(flashctx), flash_size);
 	memset(newcontents, ERASED_VALUE(flashctx), flash_size);
 
+	setup_progress_from_layout(flashctx, FLASHROM_PROGRESS_READ);
+	setup_progress_from_layout_and_diff(flashctx, curcontents, newcontents, FLASHROM_PROGRESS_ERASE);
+
 	const struct flashrom_layout *const flash_layout = get_layout(flashctx);
 	const struct romentry *entry = NULL;
 	while ((entry = layout_next_included(flash_layout, entry))) {
@@ -1616,132 +1437,7 @@ _ret:
 	return ret;
 }
 
-static int erase_by_layout(struct flashctx *const flashctx)
-{
-	if (use_legacy_erase_path)
-		return erase_by_layout_legacy(flashctx);
-	return erase_by_layout_new(flashctx);
-}
-
-static int read_erase_write_block(struct flashctx *const flashctx,
-				  const struct walk_info *const info, const erasefn_t erasefn,
-				  bool *all_skipped)
-{
-	const chipsize_t erase_len = info->erase_end + 1 - info->erase_start;
-	const bool region_unaligned = info->region_start > info->erase_start ||
-				      info->erase_end > info->region_end;
-	const uint8_t *newcontents = NULL;
-	int ret = 2;
-
-	/*
-	 * If the region is not erase-block aligned, merge current flash con-
-	 * tents into `info->curcontents` and a new buffer `newc`. The former
-	 * is necessary since we have no guarantee that the full erase block
-	 * was already read into `info->curcontents`. For the latter a new
-	 * buffer is used since `info->newcontents` might contain data for
-	 * other unaligned regions that touch this erase block too.
-	 */
-	if (region_unaligned) {
-		msg_cdbg("R");
-		uint8_t *const newc = malloc(erase_len);
-		if (!newc) {
-			msg_cerr("Out of memory!\n");
-			return 1;
-		}
-		memcpy(newc, info->newcontents + info->erase_start, erase_len);
-
-		/* Merge data preceding the current region. */
-		if (info->region_start > info->erase_start) {
-			const chipoff_t start	= info->erase_start;
-			const chipsize_t len	= info->region_start - info->erase_start;
-			if (read_flash(flashctx, newc, start, len)) {
-				msg_cerr("Can't read! Aborting.\n");
-				goto _free_ret;
-			}
-			memcpy(info->curcontents + start, newc, len);
-		}
-		/* Merge data following the current region. */
-		if (info->erase_end > info->region_end) {
-			const chipoff_t start     = info->region_end + 1;
-			const chipoff_t rel_start = start - info->erase_start; /* within this erase block */
-			const chipsize_t len      = info->erase_end - info->region_end;
-			if (read_flash(flashctx, newc + rel_start, start, len)) {
-				msg_cerr("Can't read! Aborting.\n");
-				goto _free_ret;
-			}
-			memcpy(info->curcontents + start, newc + rel_start, len);
-		}
-
-		newcontents = newc;
-	} else {
-		newcontents = info->newcontents + info->erase_start;
-	}
-
-	ret = 1;
-	bool skipped = true;
-	uint8_t *const curcontents = info->curcontents + info->erase_start;
-	const uint8_t erased_value = ERASED_VALUE(flashctx);
-	if (!(flashctx->chip->feature_bits & FEATURE_NO_ERASE) &&
-			need_erase(curcontents, newcontents, erase_len, flashctx->chip->gran, erased_value)) {
-		if (erase_block(flashctx, info, erasefn, all_skipped))
-			goto _free_ret;
-		/* Erase was successful. Adjust curcontents. */
-		memset(curcontents, erased_value, erase_len);
-		skipped = false;
-	}
-
-	unsigned int starthere = 0, lenhere = 0, writecount = 0;
-	/* get_next_write() sets starthere to a new value after the call. */
-	while ((lenhere = get_next_write(curcontents + starthere, newcontents + starthere,
-					 erase_len - starthere, &starthere, flashctx->chip->gran))) {
-		if (!writecount++)
-			msg_cdbg("W");
-		/* Needs the partial write function signature. */
-		if (write_flash(flashctx, newcontents + starthere,
-					  info->erase_start + starthere, lenhere))
-			goto _free_ret;
-		starthere += lenhere;
-		skipped = false;
-	}
-	if (skipped)
-		msg_cdbg("S");
-	else
-		*all_skipped = false;
-
-	/* Update curcontents, other regions with overlapping erase blocks
-	   might rely on this. */
-	memcpy(curcontents, newcontents, erase_len);
-	ret = 0;
-
-_free_ret:
-	if (region_unaligned)
-		free((void *)newcontents);
-	return ret;
-}
-
-/**
- * @brief Writes the included layout regions from a given image.
- *
- * If there is no layout set in the given flash context, the whole image
- * will be written.
- *
- * @param flashctx    Flash context to be used.
- * @param curcontents A buffer of full chip size with current chip contents of included regions.
- * @param newcontents The new image to be written.
- * @return 0 on success,
- *	   1 if anything has gone wrong.
- */
-static int write_by_layout_legacy(struct flashctx *const flashctx,
-			   void *const curcontents, const void *const newcontents,
-			   bool *all_skipped)
-{
-	struct walk_info info;
-	info.curcontents = curcontents;
-	info.newcontents = newcontents;
-	return walk_by_layout(flashctx, &info, read_erase_write_block, all_skipped);
-}
-
-static int write_by_layout_new(struct flashctx *const flashctx,
+static int write_by_layout(struct flashctx *const flashctx,
 			   void *const curcontents, const void *const newcontents,
 			   bool *all_skipped)
 {
@@ -1759,6 +1455,10 @@ static int write_by_layout_new(struct flashctx *const flashctx,
 		goto _ret;
 	}
 
+	setup_progress_from_layout(flashctx, FLASHROM_PROGRESS_READ);
+	setup_progress_from_layout_and_diff(flashctx, curcontents, newcontents, FLASHROM_PROGRESS_WRITE);
+	setup_progress_from_layout_and_diff(flashctx, curcontents, newcontents, FLASHROM_PROGRESS_ERASE);
+
 	const struct romentry *entry = NULL;
 	while ((entry = layout_next_included(flash_layout, entry))) {
 		ret = erase_write(flashctx, entry->region.start,
@@ -1776,15 +1476,6 @@ _ret:
 	return ret;
 }
 
-static int write_by_layout(struct flashctx *const flashctx,
-			   uint8_t *const curcontents, const uint8_t *const newcontents,
-			   bool *all_skipped)
-{
-	if (use_legacy_erase_path)
-		return write_by_layout_legacy(flashctx, curcontents, newcontents, all_skipped);
-	return write_by_layout_new(flashctx, curcontents, newcontents, all_skipped);
-}
-
 /**
  * @brief Compares the included layout regions with content from a buffer.
  *
@@ -1806,6 +1497,8 @@ static int verify_by_layout(
 {
 	const struct romentry *entry = NULL;
 
+	setup_progress_from_layout(flashctx, FLASHROM_PROGRESS_READ);
+
 	while ((entry = layout_next_included(layout, entry))) {
 		const struct flash_region *region = &entry->region;
 		const chipoff_t region_start	= region->start;
@@ -2303,6 +1996,7 @@ int flashrom_image_write(struct flashctx *const flashctx, void *const buffer, co
 		 */
 		msg_cinfo("Reading old flash chip contents... ");
 		if (verify_all) {
+			init_progress(flashctx, FLASHROM_PROGRESS_READ, flash_size);
 			if (read_flash(flashctx, oldcontents, 0, flash_size)) {
 				msg_cinfo("FAILED.\n");
 				goto _finalize_ret;
@@ -2319,12 +2013,14 @@ int flashrom_image_write(struct flashctx *const flashctx, void *const buffer, co
 
 	bool all_skipped = true;
 
+	msg_cinfo("Updating flash chip contents... ");
 	if (write_by_layout(flashctx, curcontents, newcontents, &all_skipped)) {
 		msg_cerr("Uh oh. Erase/write failed. ");
 		ret = 2;
 		if (verify_all) {
 			msg_cerr("Checking if anything has changed.\n");
 			msg_cinfo("Reading current flash chip contents... ");
+			init_progress(flashctx, FLASHROM_PROGRESS_READ, flash_size);
 			if (!read_flash(flashctx, curcontents, 0, flash_size)) {
 				msg_cinfo("done.\n");
 				if (!memcmp(oldcontents, curcontents, flash_size)) {
