@@ -148,6 +148,14 @@ enum write_granularity {
  * other flash chips, such as the ENE KB9012 internal flash, work the opposite way.
  */
 #define FEATURE_ERASED_ZERO	(1 << 18)
+/*
+ * Feature indicates that the chip does not require erase before writing:
+ * write operations can set any bit to any value without first doing an erase,
+ * but bulk erase operations may still be supported.
+ *
+ * EEPROMs usually behave this way (compare to Flash, which requires erase),
+ * for example the ST M95M02.
+ */
 #define FEATURE_NO_ERASE	(1 << 19)
 
 #define FEATURE_WRSR_EXT2	(1 << 20)
@@ -164,6 +172,11 @@ enum write_granularity {
 /* Whether chip has configuration register (RDCR/WRSR_EXT2 commands) */
 #define FEATURE_CFGR	    (1 << 25)
 
+/*
+ * Whether the chip supports serial flash hardening specified in JESD260
+ */
+#define FEATURE_FLASH_HARDENING (1 << 26)
+
 #define ERASED_VALUE(flash)	(((flash)->chip->feature_bits & FEATURE_ERASED_ZERO) ? 0x00 : 0xff)
 #define UNERASED_VALUE(flash)	(((flash)->chip->feature_bits & FEATURE_ERASED_ZERO) ? 0xff : 0x00)
 
@@ -535,12 +548,43 @@ struct flashchip {
 	 * and determines what protection range they select.
 	 */
 	enum decode_range_func decode_range;
+
+	struct rpmc_config {
+		uint8_t op1_opcode;
+		uint8_t op2_opcode;
+
+		unsigned int num_counters;
+
+		/*
+		 * Busy Polling Method :
+		 * ‘0’: Poll for OP1 busy using OP2 Extended Status[0].
+		 *      No OP1 Suspended State Support.
+		 * ‘1’: Poll for OP1 busy using Read Status (05H).
+		 *      Suspended State is supported.
+		 */
+		enum busy_polling_methods {
+			POLL_OP2_EXTENDED_STATUS = 0,
+			POLL_READ_STATUS = 1
+		} busy_polling_method;
+
+		unsigned int update_rate;
+
+		/* All times in microsecond (us) */
+		unsigned int polling_delay_read_counter_us;
+		unsigned int polling_short_delay_write_counter_us;
+		unsigned int polling_long_delay_write_counter_us;
+	} rpmc_ctx;
 };
 
 typedef int (*chip_restore_fn_cb_t)(struct flashctx *flash, void *data);
 typedef int (blockprotect_func_t)(struct flashctx *flash);
 blockprotect_func_t *lookup_blockprotect_func_ptr(const struct flashchip *const chip);
 
+struct stage_progress {
+	size_t current;
+	size_t total;
+};
+
 struct flashrom_flashctx {
 	struct flashchip *chip;
 	/* FIXME: The memory mappings should be saved in a more structured way. */
@@ -579,6 +623,10 @@ struct flashrom_flashctx {
 	/* Progress reporting */
 	flashrom_progress_callback *progress_callback;
 	struct flashrom_progress *progress_state;
+	struct stage_progress stage_progress[FLASHROM_PROGRESS_NR];
+
+	/* Maximum allowed % of redundant erase */
+	int sacrifice_ratio;
 };
 
 /* Timing used in probe routines. ZERO is -2 to differentiate between an unset
@@ -606,7 +654,6 @@ void chip_readn(const struct flashctx *flash, uint8_t *buf, const chipaddr addr,
 
 /* print.c */
 int print_supported(void);
-void print_supported_wiki(void);
 
 /* helpers.c */
 uint32_t address_to_bits(uint32_t addr);
@@ -670,6 +717,12 @@ int write_flash(struct flashctx *flash, const uint8_t *buf, unsigned int start,
  */
 #define ERROR_FLASHROM_LIMIT -201
 
+struct cli_progress {
+	unsigned int stage_pc[FLASHROM_PROGRESS_NR];
+	unsigned int visible_stages; /* Bitmask of stages with non-zero progress. */
+	bool stage_setup; /* Flag to know when to reset progress data. */
+};
+
 /* cli_common.c */
 void print_chip_support_status(const struct flashchip *chip);
 
@@ -709,7 +762,8 @@ __attribute__((format(printf, 2, 3)));
 #define msg_gspew(...)	print(FLASHROM_MSG_SPEW, __VA_ARGS__)	/* general debug spew  */
 #define msg_pspew(...)	print(FLASHROM_MSG_SPEW, __VA_ARGS__)	/* programmer debug spew  */
 #define msg_cspew(...)	print(FLASHROM_MSG_SPEW, __VA_ARGS__)	/* chip debug spew  */
-void update_progress(struct flashctx *flash, enum flashrom_progress_stage stage, size_t current, size_t total);
+void init_progress(struct flashctx *flash, enum flashrom_progress_stage stage, size_t total);
+void update_progress(struct flashctx *flash, enum flashrom_progress_stage stage, size_t increment);
 
 /* spi.c */
 struct spi_command {
@@ -723,4 +777,5 @@ int spi_send_command(const struct flashctx *flash, unsigned int writecnt, unsign
 int spi_send_multicommand(const struct flashctx *flash, struct spi_command *cmds);
 
 enum chipbustype get_buses_supported(void);
+
 #endif				/* !__FLASH_H__ */
