@@ -344,9 +344,12 @@
 #define FUDAN_FM25F01		0x3111
 #define FUDAN_FM25F02		0x3112	/* Same as FM25F02A */
 #define FUDAN_FM25F04		0x3113	/* Same as FM25F04A */
+#define FUDAN_FM25Q04		0x4013
 #define FUDAN_FM25Q08		0x4014
 #define FUDAN_FM25Q16		0x4015
 #define FUDAN_FM25Q32		0x4016
+#define FUDAN_FM25Q64		0x4017
+#define FUDAN_FM25Q128		0x4018
 
 #define FUJITSU_ID		0x04	/* Fujitsu */
 #define FUJITSU_MBM29DL400BC	0x0F
@@ -391,12 +394,16 @@
 #define GIGADEVICE_GD25Q16	0x4015	/* Same as GD25Q16B (which has OTP) */
 #define GIGADEVICE_GD25Q32	0x4016	/* Same as GD25Q32B */
 #define GIGADEVICE_GD25Q64	0x4017	/* Same as GD25Q64B */
-#define GIGADEVICE_GD25Q128	0x4018	/* Same as GD25Q128B, GD25Q127C, GD25Q128C, and GD25Q128E, can be distinguished by SFDP */
-#define GIGADEVICE_GD25Q256D	0x4019
+#define GIGADEVICE_GD25Q128	0x4018	/* Same as GD25Q128B, GD25Q127C, GD25Q128C,and GD25Q128E, GD25B128E, GD25R128E can be distinguished by SFDP */
+#define GIGADEVICE_GD25Q256D	0x4019	/* Same as GD25B256E, GD25Q256E, GD25R256E	*/
+#define GIGADEVICE_GD25B512MF	0x401A  /* Same as GD25R512MF */
 #define GIGADEVICE_GD25VQ21B	0x4212
 #define GIGADEVICE_GD25VQ41B	0x4213  /* Same as GD25VQ40C, can be distinguished by SFDP */
 #define GIGADEVICE_GD25VQ80C	0x4214
 #define GIGADEVICE_GD25VQ16C	0x4215
+#define GIGADEVICE_GD25F64F	0x4317
+#define GIGADEVICE_GD25F128F	0x4318
+#define GIGADEVICE_GD25F256F	0x4319
 #define GIGADEVICE_GD25LQ40	0x6013
 #define GIGADEVICE_GD25LQ80	0x6014
 #define GIGADEVICE_GD25LQ16	0x6015
@@ -404,9 +411,12 @@
 #define GIGADEVICE_GD25LQ64	0x6017	/* Same as GD25LQ64B (which is faster) */
 #define GIGADEVICE_GD25LQ128CD	0x6018
 #define GIGADEVICE_GD25LQ255E	0x6019
+#define GIGADEVICE_GD25LB512MF	0x601A  /* Same as GD25LR512MF */
 #define GIGADEVICE_GD25LF128E	0x6318
+#define GIGADEVICE_GD25LF256F	0x6319
+#define GIGADEVICE_GD25LF512MF	0x631A
 #define GIGADEVICE_GD25LR256E	0x6719
-#define GIGADEVICE_GD25LR512ME	0x671A
+#define GIGADEVICE_GD25LR512ME	0x671A	/*	Same as GD25LB512ME */
 #define GIGADEVICE_GD25WQ80E	0x6514
 #define GIGADEVICE_GD29GL064CAB	0x7E0601
 
@@ -534,12 +544,13 @@
 #define MACRONIX_MX25U3235E	0x2536	/* Same as MX25U6435F */
 #define MACRONIX_MX25U6435E	0x2537	/* Same as MX25U6435F */
 #define MACRONIX_MX25U12835E	0x2538	/* Same as MX25U12835F */
-#define MACRONIX_MX25U25635F	0x2539     /* Same as MX25U25643G */
+#define MACRONIX_MX25U25635F	0x2539     /* Same as MX25U25643G, MX25U25645G */
 #define MACRONIX_MX25U51245G	0x253a
 #define MACRONIX_MX25L3235D	0x5E16	/* MX25L3225D/MX25L3235D/MX25L3237D */
 #define MACRONIX_MX25L6495F	0x9517
 #define MACRONIX_MX25L3255E	0x9e16
 #define MACRONIX_MX77L25650F	0x7519
+#define MACRONIX_MX77U51250F	0x753A
 #define MACRONIX_MX25L3239E     0x2536
 
 #define MACRONIX_MX25R1635F	0x2815
@@ -845,16 +856,21 @@
 #define ST_M45PE40		0x4013
 #define ST_M45PE80		0x4014	/* Same as XM25QH80B */
 #define ST_M45PE16		0x4015
-#define XMC_XM25QH64C		0x4017
-#define XMC_XM25QU64C		0x4117
+#define XMC_XM25QH64C		0x4017	/* Same as XM25QH64D */
+#define XMC_XM25QU64C		0x4117	/* Same as XM25LU64C */
 #define XMC_XM25QU80B		0x5014
 #define XMC_XM25QH16C		0x4015	/* Same as XM25QH16D */
+#define XMC_XM25QU16C		0x5015
+#define XMC_XM25QH32C		0x4016	/* Same as XM25QH32D */
+#define XMC_XM25QU32C		0x5016
 #define XMC_XM25QH128A		0x7018
-#define XMC_XM25QH128C		0x4018
-#define XMC_XM25QU128C		0x4118
-#define XMC_XM25QH256C		0x4019
-#define XMC_XM25QU256C		0x4119
+#define XMC_XM25QH128C		0x4018	/* Same as XM25QH128D */
+#define XMC_XM25QU128C		0x4118	/* Same as XM25QU128D */
+#define XMC_XM25QH256C		0x4019	/* Same as XM25QH256D */
+#define XMC_XM25QU256C		0x4119	/* Same as XM25QU256D */
 #define XMC_XM25RU256C		0x4419
+#define XMC_XM25QH512C		0x4020	/* Same as XM25QH512D */
+#define XMC_XM25QU512C		0x4120	/* Same as XM25QU512D */
 #define ST_M25PX80		0x7114
 #define ST_M25PX16		0x7115
 #define ST_M25PX32		0x7116
@@ -1011,7 +1027,9 @@
 #define WINBOND_NEX_W25Q64_W	0x6017	/* W25Q64DW; W25Q64FV in QPI mode */
 #define WINBOND_NEX_W25Q128_W	0x6018	/* W25Q128FW; W25Q128FV in QPI mode */
 #define WINBOND_NEX_W25Q256_W	0x6019	/* W25Q256JW */
+#define WINBOND_NEX_W25R512NW	0x6020	/* W25R512NW/W74M51NW */
 #define WINBOND_NEX_W25Q16JV_M	0x7015	/* W25Q16JV_M (QE=0) */
+#define WINBOND_NEX_W25Q32JV_M	0x7016	/* W25Q32JV_M (QE=0) */
 #define WINBOND_NEX_W25Q64JV	0x7017	/* W25Q64JV */
 #define WINBOND_NEX_W25Q128_V_M	0x7018	/* W25Q128JVSM */
 #define WINBOND_NEX_W25Q256JV_M	0x7019	/* W25Q256JV_M (QE=0) */
