@@ -208,10 +208,8 @@ struct board_info {
 	const char *vendor;
 	const char *name;
 	const enum test_state working;
-#ifdef CONFIG_PRINT_WIKI
 	const char *url;
 	const char *note;
-#endif
 };
 
 extern const struct board_info boards_known[];
@@ -357,6 +355,7 @@ enum ich_chipset {
 	CHIPSET_400_SERIES_COMET_POINT,
 	CHIPSET_500_SERIES_TIGER_POINT,
 	CHIPSET_600_SERIES_ALDER_POINT,
+	CHIPSET_700_SERIES_RAPTOR_POINT,
 	CHIPSET_APOLLO_LAKE,
 	CHIPSET_GEMINI_LAKE,
 	CHIPSET_JASPER_LAKE,
@@ -527,13 +526,4 @@ static inline bool spi_chip_4ba(const struct flashctx *const flash)
 		(flash->chip->feature_bits & (FEATURE_4BA_ENTER | FEATURE_4BA_ENTER_WREN | FEATURE_4BA_ENTER_EAR7));
 }
 
-/* usbdev.c */
-struct libusb_device_handle;
-struct libusb_context;
-struct libusb_device_handle *usb_dev_get_by_vid_pid_serial(
-		struct libusb_context *usb_ctx, uint16_t vid, uint16_t pid, const char *serialno);
-struct libusb_device_handle *usb_dev_get_by_vid_pid_number(
-		struct libusb_context *usb_ctx, uint16_t vid, uint16_t pid, unsigned int num);
-
-
 #endif				/* !__PROGRAMMER_H__ */
